indexing
	description:
		"Simple version of stack, with few features and no protection %
		%against overfilling, but with carefully-written contracts."

class SIMPLE_STACK[ G ]

create
	make

feature {NONE} -- representation

	the_contents : ARRAY[ G ]

feature -- 1. Basic queries

	count : INTEGER
			-- The number of items on the stack

	item_at( i : INTEGER ) : G is
			-- The item at position 'i', where position
			-- 'count' is the top of the stack
		require
			i_big_enough:
				i >= 1
			i_small_enough:
				i <= count
		do
			Result := the_contents.item( i )
		end

feature -- 2. Derived queries

	item : G is
			-- The item on the top of the stack
		require
			stack_not_empty:
				count > 0
		do
			Result := the_contents.item( count )
		ensure
			consistent_with_item_at:
				Result = item_at( count )
		end

	is_empty : BOOLEAN is
			-- Does the stack contain no items?
		do
			Result := count = 0
		ensure
			consistent_with_count:
				Result = ( count = 0 )
		end

feature -- 3. Creation commands

	make is
			-- Make the stack empty
		do
			create the_contents.make( 1, 100 )
		ensure
			stack_is_empty:
				count = 0
		end

feature -- 4. Other commands

	put( g : G ) is
			-- Push 'g' onto the stack
		do
			count := count + 1
			the_contents.put( g, count )
		ensure
			count_increased:
				count = old count + 1
			g_on_top:
				item_at( count ) = g
		end

	remove is
			-- Delete the top item
		require
			stack_not_empty:
				count > 0
		do
			count := count - 1
		ensure
			count_decreased:
				count = old count - 1
		end


invariant
	count_is_never_negative:
		count >= 0

end -- class SIMPLE_STACK[ G ]
