indexing
	description: "Dictionaries hold keys. Each key has an associated value."

class DICTIONARY[ KEY, VALUE ]

inherit
	DBC_SUPPORT

creation
	make

feature {NONE} -- representation

	keys : ARRAY[ KEY ]
	values : ARRAY[ VALUE ]

feature -- 1. Basic queries

	count : INTEGER
			-- The number of keys in the dictionary

	has( k : KEY ) : BOOLEAN is
			-- Does the dictionary contain key 'k'?
		require
			key_exists:
				k /= Void
		local
			i : INTEGER
		do
			-- Implementation note: there is no precondition that k/=Void, so
			-- cannot do k.f without checking k.
			from
				i := 1
			until
				(i > count) or else (keys.item(i).is_equal(k))
			loop
				i := i + 1
			end
			Result := ( i <= count ) and then ( keys.item(i).is_equal(k) )
		ensure
			consistent_with_count:
				(count = 0) implies (not Result)
		end

	value_for( k : KEY ) : VALUE is
			-- The value associated with key 'k'
		require
			key_exists:
				k /= Void
			key_in_dictionary:
				has( k )
		local
			i : INTEGER
		do
			from
				i := 1
			until
				keys.item(i).is_equal(k)
			loop
				i := i + 1
			end
			Result := values.item(i)
		end				

feature -- 2. Creation commands

	make is
			-- Make an empty dictionary
		do
			!!keys.make(1, 100)
			!!values.make(1, 100)
		ensure
			dictionary_is_empty:
				count = 0
		end

feature -- 3. Other commands

	put( k : KEY; v : VALUE ) is
			-- Put key 'k' into the dictionary with associated value 'v'
		require
			key_exists:
				k /= Void
			key_not_in_dictionary:
				not has(k)
		do
			count := count + 1
			keys.put(k, count)
			values.put(v, count)
		ensure
			count_increased:
				count = old count + 1
			key_in_dictionary:
				has(k)
			value_for_k_is_v:
				value_for(k) = v
		end

	remove( k : KEY ) is
			-- Remove key 'k' from the dictionary
		require
			key_exists:
				k /= Void
			key_in_dictionary:
				has(k)
		local
			i : INTEGER
		do
			from
				i := 1
			until
				keys.item(i).is_equal(k)
			loop
				i := i + 1
			end
			keys.put( keys.item(count), i )
			values.put( values.item(count), i )
			count := count - 1
		ensure
			count_decreased:
				count = old count - 1
			key_not_in_dictionary:
				not has(k)
			value_for_k_is_undefined:
				comment( "precondition on value_for is false for argument k" )

		end

invariant
	count_never_negative:
		count >= 0
end
