indexing
	description: "Simple list of names";

class NAME_LIST

creation 
	make 

feature {NONE} -- representation

	contents : ARRAY[ STRING ]

feature {NONE} -- implementation support

	position_of( a_name : STRING ) : INTEGER is
			-- Position of 'a_name' in 'contents' or
			-- zero if 'a_name' is not in 'contents'.
		local
			index : INTEGER
		do
			from
				index := 1
			until
				index > count
			or else
				contents.item( index ).is_equal( a_name )
			loop
				index := index + 1
			end

			if
				index <= count
			then
				Result := index
			end
		end

feature  -- creation

	make is 
			-- Make a new, empty list
		do 
			create contents. make( 1, 100 )
		ensure 
			empty: count=0 
		end

feature -- basic queries

	count : INTEGER
			-- The number of names in the list

	has( a_name : STRING ) : BOOLEAN is
			-- Is name in list?
		do
			Result := position_of( a_name ) > 0
		ensure
			Result_consistent_with_count: ( count =  0 ) implies not Result
		end
			
feature -- commands

	put(a_name : STRING ) is 
			-- Add name to list
		require 
			name_not_in_list: not has( a_name ) 
		do 	
			count:=count + 1
			contents.put( a_name, count ) 
		ensure 
			count_increased: (old not has( a_name )) implies count = old count + 1
			name_in_list: (old not has( a_name )) implies has( a_name )
		end  


	remove( a_name : STRING ) is
			-- Remove name from list
		require 
			name_in_list: has( a_name ) 
		do 
			contents.put(contents.item(count), position_of( a_name ))
			count:=count -1
		ensure
			name_not_in_list: (old has( a_name )) implies not has( a_name )
			count_decreased: (old has( a_name )) implies count = old count - 1
		end

feature -- demo support

	display_contents is
			-- Display list on standard io
		local
			index : INTEGER
		do
			io.new_line
			io.put_string( "Number of names = " )
			io.put_integer( count )
			io.new_line
			from
				index := 1
			until
				index > count
			loop
				io.put_integer( index )
				io.put_string( " " )
				io.put_string( contents.item( index ) )
				io.new_line

				index := index + 1
			end
		end
end