indexing
	description:
		" An object of class CONCRETE_OBSERVER_1 is made with %
		% an object of class CONCRETE_SUBJECT, and is up-to-date %
		% with respect to that subject when its local state equals %
		% three times the state of the subject. %
		% %
		% In its bug-ridden form, this observer fails to redefine %
		% the inherited routine 'up_to_date_with_subject')."

class CONCRETE_OBSERVER_3

inherit
	ABSTRACT_OBSERVER
		redefine
			up_to_date_with_subject
		end

create
	initialize

feature -- basic queries

	my_concrete_subject : CONCRETE_SUBJECT
	my_state : INTEGER

feature -- initialization

	initialize( a_concrete_subject : CONCRETE_SUBJECT ) is
		do
			my_concrete_subject := a_concrete_subject
			my_concrete_subject.attach( Current )
		ensure
			my_concrete_subject_is_given_one:
				my_concrete_subject = a_concrete_subject
			attached_to_given_subject:
				my_concrete_subject.attached( Current )
		end

feature -- updating

	up_to_date_with_subject : BOOLEAN is
			-- Is this observer up-to-date with respect
			-- to the subject it is attached to?
		do
			if ---
				bug__concrete_observer_fails_to_redefine_is_consistent_with_subject ---
			then ---
				Result := false ---
			else ---
				Result := my_concrete_subject.concrete_state * 3 = my_state
			end ---
		ensure then
			result_is_my_state_equals_three_times_subject_state:
				Result = ( my_state = 3 * my_concrete_subject.concrete_state )
		end

	update is
			-- Bring Current up to date with the
			-- subject it is attached to
		do
			my_state := my_concrete_subject.concrete_state * 3
			io.putstring( " O3:" )
			io.putint( my_state )
			io.put_string( " " )
		ensure then
			my_state_equals_three_times_subject_state:
				my_state = 3 * my_concrete_subject.concrete_state
		end

end -- class CONCRETE_OBSERVER_3
