indexing
	description:
		"Objects of class COUNTER have a value that can be %
		%incremented and decremented. The value is an INTEGER, %
		%but can only have the values of a cardinal number (0,1,2, ...)."

class COUNTER

inherit
	SUBJECT
			-- Counter objects can have observers. Each time a counter
			-- changes its value, it calls 'notify' (from SUBJECT) so
			-- that observers can be informed that their subject has changed.
		rename
			initialize as subject_initialize
		end

create
 	initialize

feature -- basic query

	value : INTEGER
			-- The value of the counter

feature -- creation

	initialize( a_starting_value : INTEGER ) is
			-- Set 'value' to the given starting value
		require
			starting_value_not_negative:
				a_starting_value >= 0
		do
			subject_initialize
			value := a_starting_value
		ensure
			value_set_to_starting_value:
				value = a_starting_value
		end

feature -- other commands

	increment is
			-- Increment the value of the counter
		do
			value := value + 1
			notify
		ensure
			value_incremented:
				value = old value + 1
		end

	decrement is
			-- Decrement the value of the counter
		require
			value_is_positive:
				value > 0
		do
			value := value - 1
			notify
		ensure
			value_decremented:
				value = old value - 1
		end

invariant
	value_never_negative:
		value >= 0

end -- class COUNTER
