indexing
	description:
		" A main window object for the counter application %
		% has an associated counter and three controls: two %
		% buttons for incrementing and decrementing the counter %
		% and a field for displaying the counter's current value."


class MAIN_WINDOW

inherit
	WEL_FRAME_WINDOW
		redefine
			closeable
		end

create
	initialize

feature {NONE} -- initialisation

	initialize( a_counter : COUNTER ) is
			-- Remember 'a_counter'.
			-- Make top-level controls and connect everything up.
		do
			counter := a_counter
			create_main_window
			create_controls
		end

feature {NONE} -- the program's core object, a counter

	counter : COUNTER

feature {NONE}-- the main window's controls

	inc_button : INCREMENT_BUTTON
	dec_button : DECREMENT_BUTTON
	view_of_counter : VIEW_OF_COUNTER

feature {NONE} -- subroutines of initialize

	create_main_window is
		do
			make_top( "Simple counter" )
			resize( window_width, window_height )
		end

	create_controls is
			-- Create the main window's controls,
			-- attach them to the main window (by passing them 'Current'),
			-- and connect them to the counter (by passing them 'counter').
		do
			create view_of_counter.initialize(
				counter,
				Current, "",
				left_edge_of_view, gap_from_top+2,
				view_width, view_height,
				-1)

			create dec_button.initialize(
				counter,
				Current, "dec",
				left_edge_of_right_button, gap_from_top,
				button_width, button_height,
				-1)

			create inc_button.initialize(
				counter,
				Current, "inc",
				gap_from_left, gap_from_top,
				button_width, button_height,
				-1)

		end


feature -- layout constants

	-- The following attributes and functions determine
	-- the dimensions of the window and the size and
	-- position of each control within the window.

	button_width : INTEGER is 50
	button_height : INTEGER is 25

	view_width : INTEGER is 40
	view_height : INTEGER is 22

	gap_from_top : INTEGER is 20
	gap_from_left : INTEGER is 30
	gap_between_controls : INTEGER is 30

	shim_width : INTEGER is 10
		-- fudge factor (see later)

	left_edge_of_view : INTEGER is
		once
			Result :=
				gap_from_left +
				button_width +
				gap_between_controls
		end

	left_edge_of_right_button : INTEGER is
		once
			Result :=
				left_edge_of_view +
				view_width +
				gap_between_controls
		end

	window_width : INTEGER is
		once
			Result :=
				2 * gap_from_left +
					-- two times, to give same gap on right
				2 * button_width +
				view_width +
				2 * gap_between_controls +
				shim_width
					-- to give appearance of symmetry,
					-- rather than arithmetic precison
		end

	window_height : INTEGER is 90


feature -- windows infrastructure

	closeable : BOOLEAN is
			-- Get confirmation of request to close window
		do
			Result := true
				-- Main window can always be closed by user
		end

end -- class MAIN_WINDOW
