indexing
	description:
		" An object of class VIEW_OF_COUNTER can show the %
		% value of an integer counter. "

class VIEW_OF_COUNTER

inherit
	OBSERVER
		-- A VIEW_OF_COUNTER object is an observer
		-- of a COUNTER object.
create
	initialize

feature {NONE} -- representation

	the_view : WEL_SINGLE_LINE_EDIT
		-- The current implementation is based on
		-- the class SINGLE_LINE_EDIT

feature {NONE} -- creation

	initialize(a_counter : COUNTER;
		a_parent: WEL_COMPOSITE_WINDOW; a_name: STRING;
		a_x, a_y, a_width, a_height, an_id: INTEGER) is
			-- Make a widget that can show the value of the given counter.
		do
			-- Make a single line edit object, and make it unmodifiable
			create the_view.make( a_parent, a_name, a_x, a_y, a_width, a_height, an_id )
			the_view.set_read_only
			a_counter.register( Current )
			set_view_to_match_counter_value( a_counter )
		end

feature

	on_subject_changed( a_counter : COUNTER ) is
			-- Called by its subject (a counter) whenever
			-- the counter changes.
			--
			-- Show the value of the counter in the the view.
		do
			set_view_to_match_counter_value( a_counter )
		end

feature {NONE}

	set_view_to_match_counter_value( a_counter : COUNTER ) is
			-- Convert counter's value to a string and
			-- set text of view to that string.
		local
			value_as_string : STRING
		do
			create value_as_string.make(0)
			value_as_string.wipe_out
			value_as_string.append( " " )
			value_as_string.append_integer( a_counter.value )
			the_view.set_text( value_as_string )
		end


end -- class VIEW_OF_COUNTER
