indexing
	description:
		" Objects of subclasses of OBSERVER register with objects of subclasses of %
		% SUBJECT if they want to be notified that their subject has changed. %
		% %
		% (This implementation does not have the contracts introduced in Ch9) "

class SUBJECT

create
	initialize

feature {NONE} -- the list of associated observers

	the_observers : LINKED_LIST[ OBSERVER ]

feature -- creation

	initialize is
			-- Create an empty list of 'observers'
		do
			create the_observers.make
		end

feature -- receive registration

	register( an_observer : OBSERVER ) is
			-- Add 'an_observer' to the list of objects that
			-- will be notified if this subject changes.
		do
			the_observers.put_front( an_observer )
		end

feature -- send notifications

	notify is
			-- Notify all my observers that this subject has
			-- changed by calling 'update' on each one
			-- ('notify' must be called by any
			-- subclass feature that changes the state of
			-- 'Current' in a way that its observers should
			-- know about).
		do
			from
				the_observers.start
			until
				the_observers.after
			loop
				the_observers.item.on_subject_changed( Current )
				the_observers.forth
			end
		end

end -- class SUBJECT
